@@ -3,9 +3,8 @@
 from __future__ import absolute_import
 
 from storm.expr import And
-from storm.locals import Desc
 
-from . import get_db_type
+from .utils import get_db_type
 from kittystore.storm.model import Thread, Email
 
 
@@ -27,7 +26,7 @@ SQL = {
             date_active TIMESTAMP WITH TIME ZONE NOT NULL,
             PRIMARY KEY (list_name, thread_id)
         );""",
-        'CREATE INDEX "ix_thread_date_active" ON "thread" USING btree (date_active);',
+        'CREATE INDEX "ix_thread_date_active" ON "thread" (date_active);',
         'ALTER TABLE "list" ADD COLUMN "display_name" TEXT;',
         ],
     "mysql": [],
