@@ -2,7 +2,7 @@
 
 from __future__ import absolute_import
 
-from . import get_db_type
+from .utils import get_db_type
 from kittystore.storm.model import Thread
 from kittystore.analysis import compute_thread_order_and_depth
 
@@ -16,7 +16,7 @@ SQL = {
     "postgres": [
         'ALTER TABLE "email" ADD COLUMN "thread_order" INTEGER NOT NULL DEFAULT 0;',
         'ALTER TABLE "email" ADD COLUMN "thread_depth" INTEGER NOT NULL DEFAULT 0;',
-        'CREATE INDEX "ix_email_thread_order" ON "email" USING btree (thread_order);',
+        'CREATE INDEX "ix_email_thread_order" ON "email" (thread_order);',
         ],
     "mysql": [
         'ALTER TABLE `email` ADD COLUMN `thread_order` INTEGER NOT NULL DEFAULT 0;',
