@@ -19,18 +19,23 @@ from email.utils import unquote
 
 from zope.interface import implements
 from mailman.interfaces.messages import IMessageStore
+from mailman.interfaces.archiver import ArchivePolicy
 from storm.locals import Desc
-from storm.expr import And, Or, Count, Alias
+from storm.expr import And, Count, Alias
 from dateutil.tz import tzutc
 
-from kittystore import MessageNotFound
+from kittystore import MessageNotFound, events
 from kittystore.utils import parseaddr, parsedate
 from kittystore.utils import header_to_unicode
 from kittystore.scrub import Scrubber
 from kittystore.utils import get_ref_and_thread_id
 from kittystore.analysis import compute_thread_order_and_depth
 
-from .model import List, Email, Attachment, Thread, EmailFull, Category
+from .model import List, Email, Attachment, Thread, Category
+from .model import Sender, User
+
+import logging
+logger = logging.getLogger(__name__)
 
 
 class StormStore(object):
@@ -40,7 +45,7 @@ class StormStore(object):
 
     implements(IMessageStore)
 
-    def __init__(self, db, search_index=None, debug=False):
+    def __init__(self, db, search_index, settings, debug=False):
         """ Constructor.
         Create the session using the engine defined in the url.
 
@@ -50,6 +55,7 @@ class StormStore(object):
         self.db = db
         self.debug = debug
         self.search_index = search_index
+        self.settings = settings
 
 
     # IMessageStore methods
@@ -62,7 +68,7 @@ class StormStore(object):
             an X-Message-ID-Hash header, overriding any existing such
             header.
         :returns: The calculated X-Message-ID-Hash header.
-        :raises ValueError: if the message is missing a Message-ID 
+        :raises ValueError: if the message is missing a Message-ID
             header.
             The storage service is also allowed to raise this exception
             if it find, but disallows collisions.
@@ -90,20 +96,32 @@ class StormStore(object):
         l = self.db.find(List, List.name == list_name).one()
         if l is None:
             l = List(list_name)
+            # Don't wait for the cache to set those properties
+            for propname in l.mailman_props:
+                setattr(l, propname, getattr(mlist, propname))
             self.db.add(l)
-        l.display_name = mlist.display_name
-        l.subject_prefix = mlist.subject_prefix
+        if mlist.archive_policy == ArchivePolicy.never:
+            logger.info("Archiving disabled by list policy for %s" % list_name)
+            return None
         if not message.has_key("Message-Id"):
             raise ValueError("No 'Message-Id' header in email", message)
         msg_id = unicode(unquote(message['Message-Id']))
+        # Protect against extremely long Message-Ids (there is no limit in the
+        # email spec), it's set to VARCHAR(255) in the database
+        if len(msg_id) >= 255:
+            msg_id = msg_id[:254]
         email = Email(list_name, msg_id)
         if self.is_message_in_list(list_name, email.message_id):
-            print ("Duplicate email from %s: %s" %
+            logger.info("Duplicate email from %s: %s" %
                    (message['From'], message.get('Subject', '""')))
             return email.message_id_hash
 
-        # the message.as_string() call must be done before scrubbing
-        email_full = EmailFull(list_name, msg_id, message.as_string())
+        #if not getattr(settings.KITTYSTORE_FULL_EMAIL):
+        #    # If it's a valid value, leave it to the "prototype" archiver
+        #    # Note: the message.as_string() call must be done before scrubbing
+        #    email_full = EmailFull(list_name, msg_id, message.as_string())
+        #    self.db.add(email_full)
+
         # Find thread id
         new_thread = False
         ref, thread_id = get_ref_and_thread_id(message, list_name, self)
@@ -115,10 +133,18 @@ class StormStore(object):
         email.in_reply_to = ref
 
         from_name, from_email = parseaddr(message['From'])
-        from_name = header_to_unicode(from_name)
-        email.sender_name = from_name.strip()
+        from_name = header_to_unicode(from_name).strip()
         email.sender_email = unicode(from_email).strip()
+        sender = self.db.find(Sender, Sender.email == email.sender_email).one()
+        if sender is None:
+            sender = Sender(email.sender_email, from_name)
+            self.db.add(sender)
+        else:
+            sender.name = from_name # update the name if needed
         email.subject = header_to_unicode(message.get('Subject'))
+        if email.subject is not None:
+            # limit subject size to 2000 chars or PostgreSQL may complain
+            email.subject = email.subject[:2000]
         msg_date = parsedate(message.get("Date"))
         if msg_date is None:
             # Absent or unparseable date
@@ -155,16 +181,23 @@ class StormStore(object):
         self.db.add(thread)
 
         self.db.add(email)
-        self.db.add(email_full)
         compute_thread_order_and_depth(thread)
         for attachment in attachments:
             self.add_attachment(list_name, msg_id, *attachment)
         self.flush()
+        # invalidate the cache
+        events.notify(events.NewMessage(self, mlist, email))
+        if new_thread:
+            events.notify(events.NewThread(self, mlist, thread))
         # search indexing
+        # do it after caching because we need some list properties (like
+        # archive_policy)
         if self.search_index is not None:
             self.search_index.add(email)
+
         return email.message_id_hash
 
+
     def attach_to_thread(self, email, thread):
         """Attach an email to an existing thread"""
         if email.date <= thread.starting_email.date:
@@ -260,14 +293,16 @@ class StormStore(object):
         """
         msg = self.db.find(Email, And(
                     Email.list_name == unicode(list_name),
-                    Email.message_id == unicode(message_id)
+                    Email.message_id == unicode(message_id)[:254]
                 )).one()
         return msg
 
     def search(self, query, list_name=None, page=None, limit=10,
                sortedby=None, reverse=False):
-        """ Returns a list of email containing the specified keyword in
-        their content.
+        """
+        Returns a list of email corresponding to the query string. The
+        sender, subject, content and attachment names are searched. If
+        list_name is None, all public lists are searched.
 
         :param query: the query string to execute.
         :param list_name: name of the mailing list in which this email
@@ -279,76 +314,14 @@ class StormStore(object):
             by match score.
         :param reverse: reverse the order of the results.
         """
-        if list_name is not None:
-            query += " list_name:%s" % list_name
         results = self.search_index.search(
-                query, page, limit, sortedby=sortedby, reverse=reverse)
+                query, list_name, page, limit, sortedby=sortedby,
+                reverse=reverse)
         results["results"] = [ self.get_message_by_id_from_list(
                                     r["list_name"], r["message_id"])
                                for r in results["results"] ]
         return results
 
-    def search_list_for_content(self, list_name, keyword):
-        """ Returns a list of email containing the specified keyword in
-        their content.
-
-        :param list_name: name of the mailing list in which this email
-        should be searched.
-        :param keyword: keyword to search in the content of the emails.
-        """
-        emails = self.db.find(Email, And(
-                    Email.list_name == unicode(list_name),
-                    Email.content.like(u'%{0}%'.format(keyword), case_sensitive = False)
-                )).order_by(Desc(Email.date))
-        return emails
-
-    def search_list_for_content_subject(self, list_name, keyword):
-        """ Returns a list of email containing the specified keyword in
-        their content or their subject.
-
-        :param list_name: name of the mailing list in which this email
-            should be searched.
-        :param keyword: keyword to search in the content or subject of
-            the emails.
-        """
-        emails = self.db.find(Email, And(
-                    Email.list_name == unicode(list_name),
-                    Or(
-                        Email.content.like(u'%{0}%'.format(keyword), case_sensitive = False),
-                        Email.subject.like(u'%{0}%'.format(keyword), case_sensitive = False),
-                ))).order_by(Desc(Email.date))
-        return emails
-
-    def search_list_for_sender(self, list_name, keyword):
-        """ Returns a list of email containing the specified keyword in
-        the name or email address of the sender of the email.
-
-        :param list_name: name of the mailing list in which this email
-            should be searched.
-        :param keyword: keyword to search in the database.
-        """
-        emails = self.db.find(Email, And(
-                    Email.list_name == unicode(list_name),
-                    Or(
-                        Email.sender_name.like(u'%{0}%'.format(keyword), case_sensitive = False),
-                        Email.sender_email.like(u'%{0}%'.format(keyword), case_sensitive = False),
-                ))).order_by(Desc(Email.date))
-        return emails
-
-    def search_list_for_subject(self, list_name, keyword):
-        """ Returns a list of email containing the specified keyword in
-        their subject.
-
-        :param list_name: name of the mailing list in which this email
-            should be searched.
-        :param keyword: keyword to search in the subject of the emails.
-        """
-        emails = self.db.find(Email, And(
-                    Email.list_name == unicode(list_name),
-                    Email.subject.like(u'%{0}%'.format(keyword), case_sensitive = False),
-                )).order_by(Desc(Email.date))
-        return emails
-
     @property
     def messages(self):
         """An iterator over all messages in this message store."""
@@ -362,11 +335,11 @@ class StormStore(object):
         :param list_name: The fully qualified list name to which the
             message should be added.
         :param message_id: The Message-ID header contents to search for.
-        :returns: The message, or None if no matching message was found.
+        :returns: True of False (well, 1 or 0 actually)
         """
         return self.db.find(Email.message_id, And(
                     Email.list_name == unicode(list_name),
-                    Email.message_id == unicode(message_id)
+                    Email.message_id == unicode(message_id)[:254]
                 )).count()
 
 
@@ -432,7 +405,7 @@ class StormStore(object):
                     Thread.date_active >= start,
                     Thread.date_active < end,
                 )).order_by(Desc(Thread.date_active))
-        return list(threads)
+        return threads
 
     def get_start_date(self, list_name):
         """ Get the date of the first archived email in a list.
@@ -449,6 +422,21 @@ class StormStore(object):
         else:
             return None
 
+    def get_last_date(self, list_name):
+        """ Get the date of the last archived email in a list.
+
+        :param list_name: The fully qualified list name to search
+        :returns: The datetime of the last message, or None if no message have
+            been archived yet.
+        """
+        date = self.db.find(Email.date,
+                Email.list_name == unicode(list_name)
+                ).order_by(Desc(Email.date))[:1]
+        if date:
+            return date.one()
+        else:
+            return None
+
     def get_thread_neighbors(self, list_name, thread_id):
         """ Return the previous and the next threads of the specified thread,
         in date order.
@@ -526,12 +514,13 @@ class StormStore(object):
         """
         number = Alias(Count(Email.sender_email), "number")
         part = self.db.find(
-                (Email.sender_name, Email.sender_email, number),
+                (Sender.name, Email.sender_email, number),
                 And(
+                    Sender.email == Email.sender_email,
                     Email.list_name == unicode(list_name),
                     Email.date >= start,
                     Email.date < end,
-                )).group_by(Email.sender_email, Email.sender_name).order_by(Desc(number))
+                )).group_by(Email.sender_email, Sender.name).order_by(Desc(number))
         if limit is not None:
             part = part.config(limit=limit)
         return list(part)
@@ -543,6 +532,70 @@ class StormStore(object):
         return list(self.db.find(Category.name).order_by(Category.name))
 
 
+    def get_first_post(self, list_name, user_id):
+        """ Returns a user's first post on a list """
+        result = self.db.find(Email, And(
+                    Email.list_name == unicode(list_name),
+                    Email.sender_email == Sender.email,
+                    Sender.user_id == unicode(user_id),
+                    )).order_by(Email.archived_date
+                    ).config(limit=1).one()
+        return result
+
+    def get_user(self, user_id):
+        """ Returns a user given his user_id """
+        return self.db.find(User, User.id == unicode(user_id)).one()
+
+    def get_sender_name(self, user_id):
+        """ Returns a user's fullname when given his user_id """
+        result = self.db.find(Sender.name,
+                              Sender.user_id == unicode(user_id)
+                    ).config(limit=1).one()
+        return result
+
+    def _get_messages_by_user_id(self, user_id, list_name=None):
+        """ Returns a user's email hashes """
+        if list_name is None:
+            clause = And(Email.sender_email == Sender.email,
+                         Sender.user_id == unicode(user_id))
+        else:
+            clause = And(Email.sender_email == Sender.email,
+                         Sender.user_id == unicode(user_id),
+                         Email.list_name == unicode(list_name))
+        return self.db.find(Email, clause)
+
+    def get_message_hashes_by_user_id(self, user_id, list_name=None):
+        query = self._get_messages_by_user_id(user_id, list_name)
+        return list(query.values(Email.message_id_hash))
+
+    def get_message_count_by_user_id(self, user_id, list_name=None):
+        return self._get_messages_by_user_id(user_id, list_name).count()
+
+    def get_messages_by_user_id(self, user_id, list_name=None):
+        """ Returns a user's emails"""
+        query = self._get_messages_by_user_id(user_id, list_name)
+        return query.order_by(Desc(Email.date))
+
+    def get_all_messages(self):
+        return self.db.find(Email).order_by(Email.archived_date)
+
+    def get_message_dates(self, list_name, start, end):
+        """ Return all email dates between two given dates.
+
+        :param list_name: The name of the mailing list in which these emails
+            should be searched.
+        :param start: A datetime object representing the starting date of
+            the interval to query.
+        :param end: A datetime object representing the ending date of
+            the interval to query.
+        :returns: The list of messages.
+        """
+        return self.db.find(Email, And(
+                    Email.list_name == unicode(list_name),
+                    Email.date >= start,
+                    Email.date < end,
+                )).order_by(Desc(Email.date)).values(Email.date)
+
     # Attachments
 
     def add_attachment(self, mlist, msg_id, counter, name, content_type,
@@ -576,7 +629,7 @@ class StormStore(object):
         """
         att = self.db.find(Attachment, And(
                     Attachment.list_name == unicode(list_name),
-                    Attachment.message_id == unicode(message_id)
+                    Attachment.message_id == unicode(message_id)[:254]
                 )).order_by(Attachment.counter)
         return list(att)
 
@@ -591,7 +644,7 @@ class StormStore(object):
         """
         return self.db.find(Attachment, And(
                     Attachment.list_name == unicode(list_name),
-                    Attachment.message_id == unicode(message_id),
+                    Attachment.message_id == unicode(message_id)[:254],
                     Attachment.counter == counter
                 )).one()
 
