@@ -5,8 +5,8 @@
 
 import unittest
 import email
+from traceback import format_exc
 
-from mock import Mock
 from mailman.email.message import Message
 
 from kittystore.scrub import Scrubber
@@ -82,6 +82,16 @@ class TestScrubber(unittest.TestCase):
                 u"This is a test message\r\n"
                 u"Non-ASCII chars: r\xe9ponse fran\xe7ais \n")
 
+    def test_html_only_email(self):
+        # This email only has an HTML part, thus the scrubbed content will be
+        # empty. It should be an unicode empty string, not str.
+        with open(get_test_file("html-email-2.txt")) as email_file:
+            msg = email.message_from_file(email_file, _class=Message)
+        scrubber = Scrubber("testlist@example.com", msg)
+        contents, attachments = scrubber.scrub()
+        self.assertTrue(isinstance(contents, unicode),
+            u"Scrubbed content should always be unicode")
+
     def test_non_ascii_payload(self):
         """Scrubber must handle non-ascii messages"""
         for enc in ["utf8", "iso8859"]:
@@ -93,6 +103,19 @@ class TestScrubber(unittest.TestCase):
             self.assertEqual(contents, u'This message contains non-ascii '
                     u'characters:\n\xe9 \xe8 \xe7 \xe0 \xee \xef \xeb \u20ac\n')
 
+    def test_bad_content_type(self):
+        """Scrubber must handle unknown content-types"""
+        with open(get_test_file("payload-unknown.txt")) as email_file:
+            msg = email.message_from_file(email_file, _class=Message)
+        scrubber = Scrubber("testlist@example.com", msg)
+        try:
+            contents, attachments = scrubber.scrub()
+        except LookupError, e:
+            import traceback;
+            print traceback.format_exc()
+            self.fail(e) # codec not found
+        self.assertTrue(isinstance(contents, unicode))
+
     def test_attachment_4(self):
         with open(get_test_file("attachment-4.txt")) as email_file:
             msg = email.message_from_file(email_file, _class=Message)
@@ -129,6 +152,21 @@ class TestScrubber(unittest.TestCase):
                 u'accented letters : \xe9 \xe8 \xe7 \xe0.\r\nAnd an '
                 u'attachment with an accented filename\r\n\r\n\r\n\r\n')
 
+    def test_attachment_name_badly_encoded(self):
+        msg = email.message.Message()
+        msg["From"] = "dummy@example.com"
+        msg["Message-ID"] = "<dummy>"
+        msg.set_payload("Dummy content")
+        msg.add_header('Content-Disposition', 'attachment', filename='non-ascii-\xb8\xb1\xb1\xbe.jpg')
+        scrubber = Scrubber("testlist@example.com", msg)
+        try:
+            contents, attachments = scrubber.scrub()
+        except UnicodeDecodeError:
+            print format_exc()
+            self.fail("Could not decode the filename")
+        self.assertEqual(attachments,
+                [(0, 'attachment.bin', 'text/plain', None, 'Dummy content')])
+
     def test_remove_next_part_from_content(self):
         with open(get_test_file("pipermail_nextpart.txt")) as email_file:
             msg = email.message_from_file(email_file, _class=Message)
